function RDCcoords = getRDCcoords_trj(trj,prmtop,resTarget,proton,hetAtom,Dmax)
%% getRDCcoords_trj -
%   calculate average RDC coordinates and order parameters form an MD
%   trajectory
%% Inputs
%           trj - coordinates of MD trajectory, created using readmdcrd
%           prmtop - 
%           resTarget - {'VAL'}
%           proton - {'H'}
%           hetAtom - {'N'}
%
%%

% grab coordinates for spin pair of interest, e.g. 1H-15
resI = zeros(size(prmtop.residue_name,1),1);
for i = 1:size(resTarget,2)
    resI = resI | selectname(prmtop.residue_name,resTarget{i});
end

atom1 = find(selectname(prmtop.atom_name,proton{:}) & resI);
atom2 = find(selectname(prmtop.atom_name,hetAtom{:})& resI);

nSpinPairs = size(atom1,1);

% convert to one-letter amino acid code
res = aminolookup(strtrim(prmtop.residue_name(atom1,:)))';

% define variables to package into table
Residue = strcat(res,int2str(prmtop.residue_id(atom1,:)));
atom1XYZ = trj(:,to3(atom1));

% expand atom2 coordinate matrix if necessary, e.g. Val Methyls
if size(hetAtom,2) == 1
    % only one atom2
    atom2XYZ = trj(:,to3(atom2));
elseif strcmp(hetAtom{1},hetAtom{2})
    % different atom1, connected to same atom2, e.g. Val, Leu methyls
    atom2XYZ(:,to3(1:2:nSpinPairs)) = trj(:,to3(atom2));
    atom2XYZ(:,to3(2:2:nSpinPairs)) = trj(:,to3(atom2));
else
    % different atom1, connceted to different atom2, e.g. ILE Methyls
    atom2XYZ = trj(:,to3(atom2));
end
rdcVec = atom1XYZ - atom2XYZ;
meanVec = mean(rdcVec);

% calculate order parameter
% iterate over spin pairs
nRes = size(rdcVec,2)/3;
OrderParameter = zeros(nRes,1);
avg_atom1 = zeros(nRes,3);
avg_atom2 = zeros(nRes,3);
nFrames = size(rdcVec,1);
for i = 1:3:size(rdcVec,2)
    idx = (i + 2)/3; % convert loop index to vector index
    vecCols = i:i+2;
    r = vecnorm(rdcVec(:,vecCols),2,2);
    meanVecNorm1 = (meanVec(vecCols) / norm(meanVec(vecCols))); % normalize to make a unit vector
    meanVecNorm2 = meanVecNorm1 .* ones(nFrames,1); % make size consistent
    dotprod = dot(rdcVec(:,vecCols)./ r,meanVecNorm2,2);
    OrderParameter(idx) = mean((3 * dotprod.^2 - 1) / 2);
    
%     % find most representative coordinates
%     d = vecnorm(meanVec(vecCols) - rdcVec(:,vecCols),2,2);
%     [~,I] = min(d);
%     avg_atom1(idx,:) = rdcVec(I,vecCols);
%     avg_atom2(idx,:) = [0,0,0];
%     
    % save average vector
    %avg_atom1(idx,:) = meanVec(vecCols);
    avg_atom1(idx,:) = meanVecNorm1;
    avg_atom2(idx,:) = [0,0,0];
end

RDCcoords = table(string(Residue),...
    avg_atom1(:,1),avg_atom1(:,2),avg_atom1(:,3),...
    avg_atom2(:,1),avg_atom2(:,2),avg_atom2(:,3),...
    Dmax*ones(nRes,1),...
    OrderParameter,...
    'VariableNames',{'Residue','x1','y1','z1','x2','y2','z2','Dmax','Order Parameter'});

% sort by residue names, sortrows function seems to be broken for tables
[~,I] = sort(string(RDCcoords.Residue));
for i = 1:size(RDCcoords,2)
    RDCcoords(:,i) = RDCcoords(I,i);
end

end