%% importData - created by RVW 8/20/2019
% import data from user created Excel workbook

% Inputs
%   fileName - string containing the excel spreadsheet containing the
%   input data
%
% Output
%   inputData - struct of organized data

% note readcell was introduced in R2019a

function [inputData,nMedia] = importSLPdata(fileName)
[sheets] = sheetnames(fileName);

[nMedia, nPCS, nRDCPCS, ConstraintsFound, ResiduePairsFound, hsqcFound, NOEsFound, PREfound] = checkSheets(sheets);

% Load Chemical Shifts
if hsqcFound
    hsqc = readcell(fileName,'Sheet','HSQC');
    try
        inputData.hsqc = extractShifts(hsqc,'Peak','HSQC');
    catch
        inputData.hsqc = extractShifts(hsqc,'Assignment','HSQC');
    end
else
    inputData.hsqc = [];
end

% Load RDCs
if nMedia == 0
    inputData.RDCs = [];
else
    for i = 1:nMedia
        SheetName = ['RDCs ',int2str(i)];
        RDC = readcell(fileName,'Sheet',SheetName);
        inputData.RDCs(:,:,i) = extractShifts(RDC,'Peak',SheetName);
    end
end

% Load NOEs from Sparky format peak list
if NOEsFound
    NOEsparky = readcell(fileName,'Sheet','NOEs');
    try
        inputData.NOEsparky = extractShifts(NOEsparky,'Peak','NOEs');
    catch
        inputData.NOEsparky = extractShifts(NOEsparky,'Assignment','NOEs');
    end
    
    %     hsqcTol = 0.2; % distance tolerance between NOE diagonal peaks
    %     hsqcTol = 0.05;
    [inputData.SparkyStrips, inputData.NOEpeaks] = sparky2strips(inputData.NOEsparky,0.4);
else
    inputData.SparkyStrips = [];
end

% Load PREs, if present
if PREfound
    PREs = readcell(fileName,'Sheet','PREs');
    inputData.PREs = extractShifts(PREs,'Peak','PREs');
else
    inputData.PREs = [];
end

if nPCS == 0
    inputData.PCS = [];
else
    for i = 1:nPCS
        SheetName = ['PCS ',int2str(i)];
        PCS = readcell(fileName,'Sheet',SheetName);
        inputData.PCS(:,:,i) = extractShifts(PCS,'Peak',SheetName);
    end
end

if nRDCPCS == 0
    inputData.RDCPCS = [];
else
    for i =1:nRDCPCS
        SheetName = ['RDC-PCS ',int2str(i)];
        RDCPCS = readcell(fileName,'Sheet',SheetName);
        inputData.RDCPCS(:,:,i) = extractShifts(RDCPCS,'Peak',SheetName);
    end
end

% Load Constraints table, if present
if ConstraintsFound
    constraints = readcell(fileName,'Sheet','Assignment Constraints');
    constraints = extractShifts(constraints,'Peaks','Assignment Constraints');
    inputData.constraint_assignment = cell2mat(constraints(:,2:end));
else
    inputData.constraint_assignment = [];
end

if ResiduePairsFound
    residuePairs = readcell(fileName,'Sheet','Intraresidue Pairing');
    inputData.residuePairs = extractShifts(residuePairs,'Peak','Intraresidue Pairing');
else
    inputData.residuePairs = [];
end
end

function [nMedia,nPCS, nRDCPCS, ConstraintsFound, ResiduePairsFound,hsqcFound,NOEsFound,PREfound] = checkSheets(sheets)
% check that sheets are properly named and return # of RDC datasets
[rows,~] = size(sheets);

hsqcFound = false;
nMedia = 0;
nPCS = 0;
nRDCPCS = 0;
NOEsFound = false;
PREfound = false;
ConstraintsFound = false;
ResiduePairsFound = false;

for i = 1:rows
    if strcmp(sheets(i),'HSQC')
        hsqcFound = true;
    elseif strcmp(extractBetween(sheets(i),1,4),'RDCs')
        nMedia = nMedia + 1;
    elseif strcmp(sheets(i),'NOEs')
        NOEsFound = true;
    elseif strcmp(sheets(i),'PREs')
        PREfound = true;
    elseif strcmp(extractBetween(sheets(i),1,3),'PCS')
        nPCS = nPCS + 1;
    elseif strcmp(sheets(i),'Assignment Constraints')
        ConstraintsFound = true;
    elseif strcmp(sheets(i),'Intraresidue Pairing')
        ResiduePairsFound = true;
    elseif strlength(sheets(i)) > 6
        if strcmp(extractBetween(sheets(i),1,7),'RDC-PCS')
            nRDCPCS = nRDCPCS + 1;
        end
    end
end

% if ~hsqcFound
%     error('Could not find Chemical Shift data, check HSQC sheet')
% end
%
% if ~nMedia
%     error('No RDC data found, check sheet names')
% end
%
% if ~NOEsFound
%     error('No NOE data found, check sheet names')
% end
end

function data = extractShifts(C,SearchFlag,SheetName)
% This subroutine ignores any header information and extracts all cells
% in rows after a specified label

[rows,~] = size(C);
for i = 1:rows
    if strcmp(C{i,1},SearchFlag)
        if strcmp(SearchFlag,'Assignment')
            startHere = i + 2;
        else
            startHere = i + 1;
        end
    end
end

if ~exist('startHere','var')
    error(['Problem parsing ',SheetName]);
end

data = C(startHere:end,:);
end