%% newSLPanalysis
% Input
%       DataStruct - structure variable used throughout AssignSLPGUI
% Outputs
%       assignmentFrequency - matrix, each element (i,j) counts the number
%           of times peak i is assigned to residue j.
%
%       topResults -

function [assignmentFrequency,topResults,errorFlag] = newSLPanalysis(DataStruct,dataFlags)
% initialize data type flags
useCS = false; useRDC = false; useNOE = false;

for i = 1:size(dataFlags,2)
    if strcmp(dataFlags{i},'CS')
        useCS = true;
    elseif strcmp(dataFlags{i},'RDC')
        useRDC = true;
    elseif strcmp(dataFlags{i},'NOE')
        useNOE = true;
    end
end

errorFlag = []; % empty means no error

% load data, filter for unique assignments
load(DataStruct.currentFile,'GenData')

if ~isfield(GenData,'Population')
    errorFlag = 'No Assignments below Fitness Cutoff';
    assignmentFrequency = NaN;
    topResults = NaN;
    return
end

assignments = GenData.Population';
tmp = cell2mat(assignments);
[~,uniqueI,~] = unique(tmp,'rows');
assignments = assignments(uniqueI);

% calculate sub-scores and overall score
totalScore = zeros(size(assignments,1),1);
if useCS
    CS_score = chemshiftScore(assignments,DataStruct);
    totalScore = CS_score;
end
if useNOE
    NOE_score = noeScore(assignments,DataStruct);
    totalScore = totalScore + NOE_score;
end
if useRDC
    [RDC_score,back_calc] = rdcScore(assignments,DataStruct);
    totalScore = totalScore + RDC_score;
end

Penalty_score = slpPenalty(assignments,DataStruct);
totalScore = totalScore + Penalty_score;

if DataStruct.PairMethyls
    methyl_penalty = pairMethyls(assignments,DataStruct);
    totalScore = totalScore + methyl_penalty;
end

% sort everthing based on the overall score
[totalScore,scoreI] = sort(totalScore);
assignments = assignments(scoreI);

if useCS
    CS_score = CS_score(scoreI);
end
if useNOE
    NOE_score = NOE_score(scoreI);
end
if useRDC
    RDC_score = RDC_score(scoreI);
    back_calc = back_calc(scoreI,:,:);
end

if DataStruct.PairMethyls
    methyl_penalty = methyl_penalty(scoreI);
end

Penalty_score = Penalty_score(scoreI);

%% calculate assignment frequency matrix
% matrix element i,j is incremented by one if peak i is assigned to residue j
rows = size(assignments,1);
n = DataStruct.number_measurements;

assignmentFrequency=zeros(n);

for i = 1:rows
    for residue = 1:n
        assignmentFrequency(assignments{i}(residue),residue) = assignmentFrequency(assignments{i}(residue),residue) + 1;
    end
end

% convert to percent
assignmentFrequency = assignmentFrequency/rows;

% combine assignments, scores and subscores into one matrix
% topResults = [ totalScore, cell2mat(assignments), CS_score, RDC_score, NOE_score, Penalty_score,back_calc];

%% package output as a table
topResults = table(totalScore);
topResults.assignments = cell2mat(assignments);

if useCS
    topResults.CS_score = CS_score;
end

if useRDC
    topResults.RDC_score = RDC_score;
    topResults.back_calc = back_calc;
end

if useNOE
    topResults.NOE_score = NOE_score;
end

if DataStruct.PairMethyls
    topResults.Methyl_penalty = methyl_penalty;
end


topResults.Penalty_score = Penalty_score;

end