function [NOEpeakTable, NOEpairsList] = predictNOE(newPDB,CS,resTarget,proton,hetAtom,radius)
%NOEPEAKLIST produces predicted NOE peak list based upon a PDB file using
%the 1/r^6 approximation

%   Inputs
%           newPDB      edited PDB struct, created using message PDB
%
%           CS          chemical shift struct, created using readShiftX
%
%           resTarget   sparse labeling residue(s), cell array
%
%           proton      atom name, defines center of sphere for NOE calcs
%                       e.g. 'H' for amide proton
%
%           radius      cutoff distance for NOE calculation
%                       e.g. 4, for atoms within 4 angstroms of center atom
%
%   Output
%           NOEpeakList     table containing all predicted NOE crosspeaks
%                           and diagonal peaks. Note peaks without a
%                           matching chemical shift are excluded. e.g. Arg
%                           HH
%
%           NOEpairsList    table containing all predicted NOE pairs
%                           for all requested protons and their distances.
%                           Unlike NOEpeakList no protons are ignored.
%
%   Requirements
%           uses aminolookup function from bioinformatics toolbox

%% Find H atoms within cutoff radius of target atoms, calculate distance, and NOE intensity
% reshape coords for search range format
nAtoms = size(newPDB.xyz,1);
crd = reshape(newPDB.xyz',[3*nAtoms,1])';

for k = 1:size(resTarget,2)
    if ~isempty(resTarget{k})
        index_center = find(selectname(newPDB.resname,resTarget{k}) & selectname(newPDB.name,proton{k})...
            & (ismissing(cellstr(newPDB.altloc)) | selectname(newPDB.altloc,'A')));
        index_center_crd = to3(index_center);
        centerAtoms = crd(index_center_crd);
        
        % find all hydrogen atoms, exlcuding those in water molecules, and only
        % using set A if multiple coords present
        index_allH = find(selectname(newPDB.name,'H*') & ~selectname(newPDB.resname,'HOH') ...
            & (ismissing(cellstr(newPDB.altloc)) | selectname(newPDB.altloc,'A')));
        index_allH_crd = to3(index_allH);
        allH = crd(index_allH_crd);
        
        [pair, dist] = searchrange(centerAtoms,allH,radius);
        
        [pair, I] = sortrows(pair,2);
        dist = dist(I);
        
        NOEpairsList = table(newPDB.name(index_allH(pair(:,1)),:),...
            newPDB.resseq(index_allH(pair(:,1))),...
            newPDB.resname(index_allH(pair(:,1)),:),...
            newPDB.name(index_center(pair(:,2)),:),...
            newPDB.resseq(index_center(pair(:,2))),...
            newPDB.resname(index_center(pair(:,2)),:),...
            dist,...
            'VariableNames',{'RemoteH','RemoteResID','RemoteRes','CenterH','CenterResID','CenterRes','Distance'});
        
        nPairs = size(pair,1);
        %% Match Predicted NOE crosspeaks with predicted chemical shifts and create peak list
        
        NOEpeakList = cell(nPairs,7);
        count = 1;
        
        % filter out extra methyl protons
        filterRedundant = ~(selectname(NOEpairsList.RemoteRes,'VAL') & selectname(NOEpairsList.RemoteH,'HG22','HG23','HG12','HG13') | ...
            selectname(NOEpairsList.RemoteRes,'LEU') & selectname(NOEpairsList.RemoteH,'HD22','HD23','HD12','HD13') | ...
            selectname(NOEpairsList.RemoteRes,'ILE') & selectname(NOEpairsList.RemoteH,'HG12','HG13','HD12','HD13') | ...
            selectname(NOEpairsList.RemoteRes,'THR') & selectname(NOEpairsList.RemoteH,'HG22','HG23') | ...
            selectname(NOEpairsList.RemoteRes,'ALA') & selectname(NOEpairsList.RemoteH,'HB2','HB3') | ...
            selectname(NOEpairsList.RemoteRes,'MET') & selectname(NOEpairsList.RemoteH,'HE2','HE3') | ...
            selectname(NOEpairsList.RemoteRes,'LYS') & selectname(NOEpairsList.RemoteH,'HZ12','HZ13'));
        NOEpairsList = NOEpairsList(filterRedundant,:);
        nPairs = size(NOEpairsList,1);
        if nPairs == 0
            error('predictNOE: No Atoms Found!')
        end
        
        for iNOEpairs = 1:nPairs
            res = aminolookup(strtrim(NOEpairsList.RemoteRes(iNOEpairs,:)));
            resNum = NOEpairsList.RemoteResID(iNOEpairs);
            remoteH = strtrim(NOEpairsList.RemoteH(iNOEpairs,:));
            centerH = strtrim(NOEpairsList.CenterH(iNOEpairs,:));
            centerRes = aminolookup(strtrim(NOEpairsList.CenterRes(iNOEpairs,:)));
            centerResID = NOEpairsList.CenterResID(iNOEpairs,:);
            
            remoteH_idx = selectname(CS.atomName,remoteH) & CS.resID == resNum;
            centerH_idx = selectname(CS.atomName,centerH) & CS.resID == centerResID;
            
            % if no match found in CS, then skip to next iteration
            if sum(remoteH_idx) == 0
                continue
            end
            
            centerHet = hetAtom{k};
            
            centerHet_idx = selectname(CS.atomName,centerHet) & CS.resID == centerResID;
            
            NOEpeakList{count,1} = [res,int2str(resNum),'-',NOEpairsList.RemoteH(iNOEpairs,:)];
            NOEpeakList{count,4} = CS.CS(remoteH_idx);
            
            % pad residue numbers with leading zeros to allow
            % sorting later on
%             if centerResID < 10
%                 centerResIDstr = strcat('0',string(centerResID));
%             else
%                 centerResIDstr = string(centerResID);
%             end
            maxDigits = 3;
            padStr = num2str(zeros(1,maxDigits - 1 - floor(log10(centerResID))),'%d');
            
            NOEpeakList{count,2} = strcat(centerRes,padStr,string(centerResID),'-',centerHet);
            NOEpeakList{count,5} = CS.CS(centerHet_idx);
            
            NOEpeakList{count,3} = NOEpairsList.CenterH(iNOEpairs,:);
            NOEpeakList{count,6} = CS.CS(centerH_idx);
            
            % calculate NOE intensity based on proton-proton distance
            NOEpeakList{count,7} = 100*NOEpairsList.Distance(iNOEpairs)^-6;
            count = count + 1;
            
        end
        
        % trim empty cells
        NOEpeakList = NOEpeakList(1:count-1,:);        
        
        if k == 1
            tmpC = NOEpeakList;
        else
            tmpC = [tmpC;NOEpeakList];
        end
    end
end
% sort by second column, e.g. "V05-N"
[~,I] = sort(string(tmpC(:,2)));
tmpC = tmpC(I,:);

% insert empty rows
lastRes = tmpC{1,2};
emptyRow = cell(1,7);
tmpC = [emptyRow;tmpC];

nRes = size(unique([tmpC{:,2}]),2);
for i = 2:size(tmpC,1)+nRes-1
    currentRes = tmpC{i,2};
    if ~strcmp(currentRes,lastRes)
        tmpC = [tmpC(1:i-1,:);emptyRow;tmpC(i:end,:)];
        lastRes = currentRes;
    end
end
        
NOEpeakTable = table(tmpC(:,1),...
    tmpC(:,2),...
    tmpC(:,3),...
    tmpC(:,4),...
    tmpC(:,5),...
    tmpC(:,6),...
    tmpC(:,7),...
    'VariableNames',{'Assignment','Center Heteroatom','Center Proton','w1','w2','w3','Intensity'});
