function [NOEpeakTable, NOEpairsList,R_all,NOE] = predictNOE_trj(trj,prmtop,CS,resTarget,proton,hetAtom,parms)
%NOEPEAKLIST produces predicts NOE peak list from an MD trajectory
%   Inputs
%           trj         MD trajectory coordinates, created using readmdcrd,
%                       mdtoolbox function
%
%           prmtop      topology file, created using readprmtop, mdtoolbox
%
%           CS          chemical shift struct, created using readShiftX
%
%           resTarget   sparse labeling residue(s), cell array
%
%           proton      atom name, defines center of sphere for NOE calcs
%                       e.g. 'H' for amide proton
%
%           parms       struct containing several parameters for the
%                       calculation
%
%               .radius     cutoff distance for NOE calculation
%                            e.g. 4, for atoms within 4 angstroms of center atom
%               
%               .tauC       rotational tumbling time, ns
%               
%               .tMax       maximum mixing time for NOE calculation, ms
%
%               .tStep      time step of MD trajectory frames, s
%
%               .Hfreq      Proton Larmor frequency, MHz
%
%               .ref_frame  reference frame for applying distance cutoff,
%               scalar double
%   Output
%           NOEpeakList     table containing all predicted NOE crosspeaks
%                           and diagonal peaks. Note peaks without a
%                           matching chemical shift are excluded. e.g. Arg
%                           HH
%
%           NOEpairsList    table containing all predicted NOE pairs
%                           for all requested protons and their distances.
%                           Unlike NOEpeakList no protons are ignored.  
%
%   Requirements
%           uses aminolookup function from bioinformatics toolbox

%% initialize
wH = parms.Hfreq * 1e6 * 2*pi; % proton larmor frequency, rad per sec
% dd = 7.549e5; % dipolar constant
d00 = dipolarC();
maxFreq = 2*pi/(2*parms.tStep); 

tMix = linspace(0,parms.tMax)*1e-3; % seconds
tauC = parms.tauC*1e-9; % seconds

% internal correlation window size
nFrames = size(trj,1);
segSize = 3*tauC;
segFrames = floor(segSize/parms.tStep);
segHalf = floor(segFrames/2);
nSegs = floor(nFrames/segFrames);

% ensure at least 1000 points ultimately included in the spectral density
if segHalf < 1000
    segHalf = 1000;
    segFrames = segHalf * 2;
    nSegs = floor(nFrames/segFrames);
end

fprintf('Splitting trajectory into %d segments of %d frames...\n',nSegs,segFrames)

% indices to sample spectral density
freqStep = maxFreq/segHalf;
w2tp =ceil(2*wH/freqStep) + 1; 
wtp = ceil(wH/freqStep) + 1;

%% Find H atoms within cutoff radius of target atoms, calculate distance, and NOE intensity

index_center = find(selectname(prmtop.residue_name,resTarget{:}) & selectname(prmtop.atom_name,proton{:}));
centerAtoms = trj(parms.ref_frame,to3(index_center));

% find all hydrogen atoms, exlcuding those in water molecules, and only
% using set A if multiple coords present
index_allH = find(selectname(prmtop.atom_name,'H*') & ~selectname(prmtop.residue_name,'WAT'));
allH = trj(parms.ref_frame,to3(index_allH));

[pair, dist] = searchrange_exhaustive(centerAtoms,allH,parms.radius);

[pair, I] = sortrows(pair,2);
dist = dist(I);

remoteH_id = prmtop.atom_id(index_allH(pair(:,1)));
centerH_id = prmtop.atom_id(index_center(pair(:,2)));

centerH_unique = unique(centerH_id);

NOEpairsList = table(prmtop.atom_name(index_allH(pair(:,1)),:),...
                    prmtop.residue_id(index_allH(pair(:,1))),...
                    prmtop.residue_name(index_allH(pair(:,1)),:),...
                    prmtop.atom_name(index_center(pair(:,2)),:),...
                    prmtop.residue_id(index_center(pair(:,2))),...
                    prmtop.residue_name(index_center(pair(:,2)),:),...
                    dist,...
                    'VariableNames',{'RemoteH','RemoteResID','RemoteRes','CenterH','CenterResID','CenterRes','Distance'});

totalPairs = size(pair,1);
%% Internal Correlation Functions
% iterate over residues
R_all = cell(1,size(centerH_unique,1));
lastPair = 0;
NOE = zeros(totalPairs,100);
for i = 1:size(centerH_unique,1)
    nPairs = size(centerH_id(centerH_id == centerH_unique(i)),1);
    R = zeros(nPairs);
    d = zeros(nPairs);
    
    invertedSpin = remoteH_id(lastPair+1:lastPair+nPairs) == centerH_unique(i);
    
    fprintf('Starting Residue %d, with %d spins\n',i,nPairs);
    tic
    % iterate over spin pairs
    for j = 1:nPairs
        jj = lastPair + j;
        for k = j+1:nPairs
            kk = lastPair + k;
            atom1_id = remoteH_id(jj);
            atom2_id = remoteH_id(kk);
            
            atom1 = trj(:,to3(atom1_id));
            atom2 = trj(:,to3(atom2_id));
            
            % skip pair if too far apart in first frame
            d(j,k) = norm(atom1(1,:) - atom2(1,:));
            if  d(j,k) > parms.radius
                continue
            end
            
            %% start internal correlation function calculation
            % loop over subsegments
            c = zeros(segHalf,1); % correlation function for all segments
            
            parfor segs = 1:nSegs
                currentFrames = (segs-1)*segFrames+1 : segs*segFrames;
                
                % interaction vector
                v = atom1(currentFrames,:) - atom2(currentFrames,:);

                c_seg = zeros(segHalf,1);
                for t = 1:segHalf
                    v_t = v(t,:) .* ones(segHalf,1);
                    r_t = norm(v_t(1,:));

                    tau = t + (1:segHalf);
                    v_tau = v(tau,:);
                    r_tau = vecnorm(v_tau,2,2);

                    cosTheta = dot(v_t,v_tau,2);
                    cosTheta = cosTheta / r_t;
                    cosTheta = cosTheta ./ r_tau;

                    p20 = (3*cosTheta.^2 - 1)/2;
                    p20 = p20 .* (r_t .* r_tau).^-3;

                    c_seg = c_seg + p20;
                end
                c = c + c_seg;
            end
            c = c/segHalf;
            c = c/nSegs;
            
            x = 1:segHalf;
            c = c' .* exp(-x*parms.tStep/tauC);
            J = dct(c,'Type',3);
            npts = length(J);
            J = J * pi / sqrt(2*npts) * npts / maxFreq;
            
            % now calculate relaxation rate matrix elements
            % equations from Cavanagh, pp377
            R(j,k) = (d00/10)*(-J(1)+6*J(w2tp)); % sigma
            R(k,j) = R(j,k);
            
            rho = (d00/10)*(J(1)+3*J(wtp)+6*J(w2tp));
            R(j,j) = R(j,j) + rho;
            R(k,k) = R(k,k) + rho;
        end
    end
    R_all{i} = R;
    
    % define equilibrium state 
    Ieq = ones(nPairs,1);
    Iinit = Ieq;
    Iinit(invertedSpin) = -1;
    % Integrate system of differential equations
    y0 = (Iinit-Ieq)';
    [t,y] = ode45(@(t,y)odefun(t,y,R),tMix,y0);
    NOE(lastPair+1:lastPair+nPairs,:) = y';
    
    timeTake(i) = toc;
    lastPair = lastPair + nPairs;
end
fprintf('Finished in %.0f minutes %.0f seconds\n',sum(timeTake)/60,mod(sum(timeTake),60));

NOEpairsList.NOEs = -NOE;
%% Match Predicted NOE crosspeaks with predicted chemical shifts and create peak list

NOEpeakList = cell(totalPairs,7);
count = 1;

% rename atoms to match CS file
[NOEpairsList.RemoteRes, NOEpairsList.RemoteH] = messageMD(NOEpairsList.RemoteRes,NOEpairsList.RemoteH);

for iNOEpairs = 1:totalPairs
    res = aminolookup(strtrim(NOEpairsList.RemoteRes(iNOEpairs,:)));
    resNum = NOEpairsList.RemoteResID(iNOEpairs);
    atom1 = strtrim(NOEpairsList.RemoteH(iNOEpairs,:));
    atom2 = strtrim(NOEpairsList.CenterH(iNOEpairs,:));
    centerRes = aminolookup(strtrim(NOEpairsList.CenterRes(iNOEpairs,:)));
    centerResID = NOEpairsList.CenterResID(iNOEpairs,:);
    
    remoteH_idx = selectname(CS.atomName,atom1) & CS.resID == resNum;
    centerH_idx = selectname(CS.atomName,atom2) & CS.resID == centerResID;
    
    % if no match found in CS, then skip to next iteration
    if sum(remoteH_idx) == 0
        continue
    end
    
    % determine appropriate heteronucleus, e.g. CG1 vs CG2
    for j = 1:size(proton,2)
        if strcmp(atom2,proton{j})
           centerHet = hetAtom{j}; 
        end
    end
    
    centerHet_idx = selectname(CS.atomName,centerHet) & CS.resID == centerResID;
       
    NOEpeakList{count,1} = [res,int2str(resNum),'-',NOEpairsList.RemoteH(iNOEpairs,:)];
    NOEpeakList{count,4} = CS.CS(remoteH_idx);

    % pad residue numbers below 10 with leading zero to allow
    % sorting later on
    if centerResID < 10
        centerResIDstr = strcat('00',string(centerResID));
    elseif centerResID < 100
        centerResIDstr = strcat('0',string(centerResID));
    else
        centerResIDstr = string(centerResID);
    end

    NOEpeakList{count,2} = strcat(centerRes,centerResIDstr,'-',centerHet);
    NOEpeakList{count,5} = CS.CS(centerHet_idx);
   
    NOEpeakList{count,3} = NOEpairsList.CenterH(iNOEpairs,:);
    NOEpeakList{count,6} = CS.CS(centerH_idx);
        
    % calculate NOE intensity based on proton-proton distance
    NOEpeakList{count,7} = NOEpairsList.NOEs(iNOEpairs,end);
    count = count + 1;
    
end

% trim empty cells
NOEpeakList = NOEpeakList(1:count-1,:);

% sort by second column
[~,I] = sort(string(NOEpeakList(:,2)));
NOEpeakList = NOEpeakList(I,:);

% insert empty rows
lastRes = NOEpeakList{1,2};
emptyRow = cell(1,7);
NOEpeakList = [emptyRow;NOEpeakList];

nRes = size(unique(NOEpairsList.CenterResID),1);
for i = 2:size(NOEpeakList,1)+nRes-1
    currentRes = NOEpeakList{i,2};
    if ~strcmp(currentRes,lastRes)
        NOEpeakList = [NOEpeakList(1:i-1,:);emptyRow;NOEpeakList(i:end,:)];
        lastRes = currentRes;
    end
end

NOEpeakTable = table(NOEpeakList(:,1),...
                    NOEpeakList(:,2),...
                    NOEpeakList(:,3),...
                    NOEpeakList(:,4),...
                    NOEpeakList(:,5),...
                    NOEpeakList(:,6),...
                    NOEpeakList(:,7),...
                    'VariableNames',{'Assignment','Center Heteroatom','Center Proton','w1','w2','w3','Intensity'});

end

function dydt = odefun(t,y,R)
nPairs=size(R,1);
dydt = zeros(nPairs,1);
dydt(:) = -R(:,:)*y(:);
end

function d00 = dipolarC()
% calculate dipolar constant
gammaH = 2.6752e8; % per T per s
hbar = (6.626e-34)/(2*pi); % J-s
mu0 = 4*pi*1e-7; % T^2 m^3 per J

d00 = (mu0/(4*pi))^2 * gammaH^4 * hbar^2;% m^6 s^-2
d00 = d00 * 1e60; % convert m to Angstroms
end