function [NOEpeakTable, NOEpairsList,R_all,NOE] = predictNOE_trj(trj,prmtop,CS,resTarget,proton,hetAtom,parms,factor)
%NOEPEAKLIST produces predicts NOE peak list from an MD trajectory
%   Inputs
%           trj         MD trajectory coordinates, created using readmdcrd,
%                       mdtoolbox function
%
%           prmtop      topology file, created using readprmtop, mdtoolbox
%
%           CS          chemical shift struct, created using readShiftX
%
%           resTarget   sparse labeling residue(s), cell array
%
%           proton      atom name, defines center of sphere for NOE calcs
%                       e.g. 'H' for amide proton
%
%           parms       struct containing several parameters for the
%                       calculation
%
%               .radius     cutoff distance for NOE calculation
%                            e.g. 4, for atoms within 4 angstroms of center atom
%               .tauC
%               .tMax
%               .tStep
%               .Hfreq
%
%   Output
%           NOEpeakList     table containing all predicted NOE crosspeaks
%                           and diagonal peaks. Note peaks without a
%                           matching chemical shift are excluded. e.g. Arg
%                           HH
%
%           NOEpairsList    table containing all predicted NOE pairs
%                           for all requested protons and their distances.
%                           Unlike NOEpeakList no protons are ignored.  
%
%   Requirements
%           uses aminolookup function from bioinformatics toolbox

%% initialize
wH = parms.Hfreq * 1e6 *2*pi; % proton larmor frequency, Hz
% dd = 7.549e5; % dipolar constant
d00 = dipolarC();
maxFreq = 1/(2*parms.tStep); 

tMix = linspace(0,parms.tMax)*1e-3;
tauC = parms.tauC*1e-9;

% internal correlation window size
nFrames = size(trj,1);
segHalf = nFrames;

% indices to sample spectral density
freqStep = maxFreq/segHalf;
w2tp =ceil(2*wH/freqStep); 
wtp = ceil(wH/freqStep);

%% Find H atoms within cutoff radius of target atoms, calculate distance, and NOE intensity

index_center = find(selectname(prmtop.residue_name,resTarget{:}) & selectname(prmtop.atom_name,proton{:}));
centerAtoms = trj(1,to3(index_center));

% find all hydrogen atoms, exlcuding those in water molecules, and only
% using set A if multiple coords present
index_allH = find(selectname(prmtop.atom_name,'H*') & ~selectname(prmtop.residue_name,'WAT'));
allH = trj(1,to3(index_allH));

[pair, dist] = searchrange(centerAtoms,allH,parms.radius);

[pair, I] = sortrows(pair,2);
dist = dist(I);

remoteH_id = prmtop.atom_id(index_allH(pair(:,1)));
centerH_id = prmtop.atom_id(index_center(pair(:,2)));

centerH_unique = unique(centerH_id);

NOEpairsList = table(prmtop.atom_name(index_allH(pair(:,1)),:),...
                    prmtop.residue_id(index_allH(pair(:,1))),...
                    prmtop.residue_name(index_allH(pair(:,1)),:),...
                    prmtop.atom_name(index_center(pair(:,2)),:),...
                    prmtop.residue_id(index_center(pair(:,2))),...
                    prmtop.residue_name(index_center(pair(:,2)),:),...
                    dist,...
                    'VariableNames',{'RemoteH','RemoteResID','RemoteRes','CenterH','CenterResID','CenterRes','Distance'});

totalPairs = size(pair,1);
%% Internal Correlation Functions
% iterate over residues
R_all = cell(1,size(centerH_unique,1));
lastPair = 0;
NOE = zeros(totalPairs,100);
for i = 1:size(centerH_unique,1)
    nPairs = size(centerH_id(centerH_id == centerH_unique(i)),1);  

    % check for equivalent spins
    clusterXYZ = zeros(nFrames,nPairs*3);
    mult = zeros(nPairs,1);
    excludeList = [];
    for j = 1:nPairs
        current_id = remoteH_id(lastPair+j);
        
        remoteH_name = prmtop.atom_name(current_id,:);
        remoteH_res = strtrim(prmtop.residue_name(current_id,:));
        remoteH_resID = prmtop.residue_id(current_id);
        
        cols = 3*j-2:3*j;
        % check for excluded spins
        if ~any(current_id == excludeList)
            if strcmp(remoteH_res,'VAL')
                if strcmp(remoteH_name(1:3),'HG1')
                    % create methyl pseudoatom with multiplicity of 3
                    methyl_ID = selectid(prmtop.residue_id,remoteH_resID) & selectname(prmtop.atom_name,'HG11','HG12','HG13');
                    excludeList = [excludeList;find(methyl_ID)];
                    trjCols = find(to3(methyl_ID));
                    
                    clusterXYZ(:,cols) = [mean([trj(:,trjCols(1)),trj(:,trjCols(4)),trj(:,trjCols(7))],2), ...
                        mean([trj(:,trjCols(2)),trj(:,trjCols(5)),trj(:,trjCols(8))],2), ...
                        mean([trj(:,trjCols(3)),trj(:,trjCols(6)),trj(:,trjCols(9))],2)];
                    
                    mult(j) = 3;
                    
                elseif strcmp(remoteH_name(1:3),'HG2')
                    methyl_ID = selectid(prmtop.residue_id,remoteH_resID) & selectname(prmtop.atom_name,'HG21','HG22','HG23');
                    excludeList = [excludeList;find(methyl_ID)];
                    trjCols = find(to3(methyl_ID));
                    
                    clusterXYZ(:,cols) = [mean([trj(:,trjCols(1)),trj(:,trjCols(4)),trj(:,trjCols(7))],2), ...
                        mean([trj(:,trjCols(2)),trj(:,trjCols(5)),trj(:,trjCols(8))],2), ...
                        mean([trj(:,trjCols(3)),trj(:,trjCols(6)),trj(:,trjCols(9))],2)];
                    mult(j) = 3;
                else
                    % grab coordinates without manipulation
                    trjCols = to3(current_id);
                    clusterXYZ(:,cols) = trj(:,trjCols);
                    
                    mult(j) = 1;
                end
            elseif strcmp(remoteH_res,'LEU')
                if strcmp(remoteH_name(1:3),'HD1')
                    methyl_ID = selectid(prmtop.residue_id,remoteH_resID) & selectname(prmtop.atom_name,'HD11','HD12','HD13');
                    excludeList = [excludeList;find(methyl_ID)];
                    trjCols = find(to3(methyl_ID));
                    
                    clusterXYZ(:,cols) = [mean([trj(:,trjCols(1)),trj(:,trjCols(4)),trj(:,trjCols(7))],2), ...
                        mean([trj(:,trjCols(2)),trj(:,trjCols(5)),trj(:,trjCols(8))],2), ...
                        mean([trj(:,trjCols(3)),trj(:,trjCols(6)),trj(:,trjCols(9))],2)];
                    
                    mult(j) = 3;
                elseif strcmp(remoteH_name(1:3),'HD2')
                    methyl_ID = selectid(prmtop.residue_id,remoteH_resID) & selectname(prmtop.atom_name,'HD21','HD22','HD23');
                    excludeList = [excludeList;find(methyl_ID)];
                    trjCols = find(to3(methyl_ID));
                    
                    clusterXYZ(:,cols) = [mean([trj(:,trjCols(1)),trj(:,trjCols(4)),trj(:,trjCols(7))],2), ...
                        mean([trj(:,trjCols(2)),trj(:,trjCols(5)),trj(:,trjCols(8))],2), ...
                        mean([trj(:,trjCols(3)),trj(:,trjCols(6)),trj(:,trjCols(9))],2)];
                    
                    mult(j) = 3;
                else
                    % grab coordinates without manipulation
                    trjCols = to3(current_id);
                    clusterXYZ(:,cols) = trj(:,trjCols);
                    
                    mult(j) = 1;
                end
            elseif strcmp(remoteH_res,'ILE')
                if strcmp(remoteH_name(1:3),'HD1')
                    methyl_ID = selectid(prmtop.residue_id,remoteH_resID) & selectname(prmtop.atom_name,'HD11','HD12','HD13');
                    excludeList = [excludeList;find(methyl_ID)];
                    trjCols = find(to3(methyl_ID));
                    
                    clusterXYZ(:,cols) = [mean([trj(:,trjCols(1)),trj(:,trjCols(4)),trj(:,trjCols(7))],2), ...
                        mean([trj(:,trjCols(2)),trj(:,trjCols(5)),trj(:,trjCols(8))],2), ...
                        mean([trj(:,trjCols(3)),trj(:,trjCols(6)),trj(:,trjCols(9))],2)];
                    
                    mult(j) = 3;
                elseif strcmp(remoteH_name(1:3),'HG2')
                    methyl_ID = selectid(prmtop.residue_id,remoteH_resID) & selectname(prmtop.atom_name,'HG21','HG22','HG23');
                    excludeList = [excludeList;find(methyl_ID)];
                    trjCols = find(to3(methyl_ID));
                    
                    clusterXYZ(:,cols) = [mean([trj(:,trjCols(1)),trj(:,trjCols(4)),trj(:,trjCols(7))],2), ...
                        mean([trj(:,trjCols(2)),trj(:,trjCols(5)),trj(:,trjCols(8))],2), ...
                        mean([trj(:,trjCols(3)),trj(:,trjCols(6)),trj(:,trjCols(9))],2)];
                    
                    mult(j) = 3;
                else
                    % grab coordinates without manipulation
                    trjCols = to3(current_id);
                    clusterXYZ(:,cols) = trj(:,trjCols);
                    
                    mult(j) = 1;
                end
            elseif strcmp(remoteH_res,'THR')
                if strcmp(remoteH_name(1:3),'HG2')
                    methyl_ID = selectid(prmtop.residue_id,remoteH_resID) & selectname(prmtop.atom_name,'HG21','HG22','HG23');
                    excludeList = [excludeList;find(methyl_ID)];
                    trjCols = find(to3(methyl_ID));
                    
                    clusterXYZ(:,cols) = [mean([trj(:,trjCols(1)),trj(:,trjCols(4)),trj(:,trjCols(7))],2), ...
                        mean([trj(:,trjCols(2)),trj(:,trjCols(5)),trj(:,trjCols(8))],2), ...
                        mean([trj(:,trjCols(3)),trj(:,trjCols(6)),trj(:,trjCols(9))],2)];
                    
                    mult(j) = 3;
                else
                    % grab coordinates without manipulation
                    trjCols = to3(current_id);
                    clusterXYZ(:,cols) = trj(:,trjCols);
                    
                    mult(j) = 1;
                end
            elseif strcmp(remoteH_res,'ALA')
                if strcmp(remoteH_name(1:2),'HB')
                    methyl_ID = selectid(prmtop.residue_id,remoteH_resID) & selectname(prmtop.atom_name,'HB1','HB2','HB3');
                    excludeList = [excludeList;find(methyl_ID)];
                    trjCols = find(to3(methyl_ID));
                    
                    clusterXYZ(:,cols) = [mean([trj(:,trjCols(1)),trj(:,trjCols(4)),trj(:,trjCols(7))],2), ...
                        mean([trj(:,trjCols(2)),trj(:,trjCols(5)),trj(:,trjCols(8))],2), ...
                        mean([trj(:,trjCols(3)),trj(:,trjCols(6)),trj(:,trjCols(9))],2)];
                    
                    mult(j) = 3;
                else
                    % grab coordinates without manipulation
                    trjCols = to3(current_id);
                    clusterXYZ(:,cols) = trj(:,trjCols);
                    
                    mult(j) = 1;
                end
            elseif strcmp(remoteH_res,'MET')
                if strcmp(remoteH_name(1:2),'HE')
                    methyl_ID = selectid(prmtop.residue_id,remoteH_resID) & selectname(prmtop.atom_name,'HE1','HE2','HE3');
                    excludeList = [excludeList;find(methyl_ID)];
                    trjCols = find(to3(methyl_ID));
                    
                    clusterXYZ(:,cols) = [mean([trj(:,trjCols(1)),trj(:,trjCols(4)),trj(:,trjCols(7))],2), ...
                        mean([trj(:,trjCols(2)),trj(:,trjCols(5)),trj(:,trjCols(8))],2), ...
                        mean([trj(:,trjCols(3)),trj(:,trjCols(6)),trj(:,trjCols(9))],2)];
                    
                    mult(j) = 3;
                else
                    % grab coordinates without manipulation
                    trjCols = to3(current_id);
                    clusterXYZ(:,cols) = trj(:,trjCols);
                    
                    mult(j) = 1;
                end
            else
                % grab coordinates without manipulation
                trjCols = to3(current_id);
                clusterXYZ(:,cols) = trj(:,trjCols);
                
                mult(j) = 1;
            end
        end
    end
    
    % filter out empty columns, from excluded atoms
    test = clusterXYZ == zeros(nFrames,1);    
    clusterXYZ(:,(test(1,:))) = [];
    clear test
    
    trimList = mult == 0;
    mult = mult(mult~=0);
    
    newPairs= size(clusterXYZ,2)/3;
    R = zeros(newPairs);
    d = zeros(newPairs);    
    
    newList = remoteH_id(lastPair+1:lastPair+nPairs);
    newList = newList(~trimList);
    
    % find center atom          
    invertedSpin = newList == centerH_unique(i);
    
    fprintf('Starting Residue %d, with %d spins\n',i,newPairs);
    tic
    % iterate over spin pairs
    for j = 1:newPairs
        for k = j+1:newPairs                        
            atom1 = clusterXYZ(:,to3(j));
            atom2 = clusterXYZ(:,to3(k));
            
            % skip pair if too far apart in first frame
            d(j,k) = norm(atom1(1,:) - atom2(1,:));
            if  d(j,k) > parms.radius
                continue
            end
            
            %% start internal correlation function calculation
            % interaction vector
            v = atom1 - atom2;
            v0 = v(1,:) .* ones(nFrames,1);
            
            % average inter-spin distance, to the inverse sixth power
            r6 = mean(vecnorm(v,2,2).^-3)^2;
            r6 = r6 * 1e60; % convert from ansgroms^-6 to m^-6
            
            % convert to unit vectors
            v = v ./ vecnorm(v,2,2);
            v0 = v0 ./ vecnorm(v0,2,2);
            
            cosTheta = dot(v0,v,2);
            p20 = (3*cosTheta.^2 -1)/2;
            c = xcorr(p20,'normalized');
            c = c(nFrames:end);
            x = 1:nFrames;
            c = c' .* exp(-x*parms.tStep/tauC);
            J = dct(c);
            
            % normalize J
            J = J * tauC / J(1);
            
            % now calculate relaxation rate matrix elements
            % equations from Cavanagh, pp377
            R(j,k) = (d00*r6/4)*(-J(1)+6*J(w2tp)); %* mult(j) * mult(k); % sigma
            R(k,j) = R(j,k);
            
            rho = (d00*r6/4)*(J(1)+3*J(wtp)+6*J(w2tp));
            R(j,j) = R(j,j) + rho;
            R(k,k) = R(k,k) + rho;
        end
    end
    R = R/factor;
    R_all{i} = R;
    
    % define equilibrium state 
    Ieq = ones(newPairs,1);
    Iinit = Ieq;
    Iinit(invertedSpin) = -1;
    % Integrate system of differentail equations
    y0 = (Iinit-Ieq)';
    [t,y] = ode45(@(t,y)odefun(t,y,R),tMix,y0);
    NOE(lastPair+1:lastPair+newPairs,:) = y';
    
    timeTake(i) = toc;
    lastPair = lastPair + nPairs;
end
fprintf('Finished in %.0f minutes %.0f seconds\n',sum(timeTake)/60,mod(sum(timeTake),60));

NOEpairsList.NOEs = -NOE(:,end);
%% Match Predicted NOE crosspeaks with predicted chemical shifts and create peak list

NOEpeakList = cell(totalPairs,7);
count = 1;

for iNOEpairs = 1:totalPairs
    res = aminolookup(strtrim(NOEpairsList.RemoteRes(iNOEpairs,:)));
    resNum = NOEpairsList.RemoteResID(iNOEpairs);
    atom1 = strtrim(NOEpairsList.RemoteH(iNOEpairs,:));
    atom2 = strtrim(NOEpairsList.CenterH(iNOEpairs,:));
    centerRes = aminolookup(strtrim(NOEpairsList.CenterRes(iNOEpairs,:)));
    centerResID = NOEpairsList.CenterResID(iNOEpairs,:);
    
    remoteH_idx = selectname(CS.atomName,atom1) & CS.resID == resNum;
    centerH_idx = selectname(CS.atomName,atom2) & CS.resID == centerResID;
    
    % if no match found in CS, then skip to next iteration
    if sum(remoteH_idx) == 0
        continue
    end
    
    % determine appropriate heteronucleus, e.g. CG1 vs CG2
    for j = 1:size(proton,2)
        if strcmp(atom2,proton{j})
           centerHet = hetAtom{j}; 
        end
    end
    
    centerHet_idx = selectname(CS.atomName,centerHet) & CS.resID == centerResID;
       
    NOEpeakList{count,1} = [res,int2str(resNum),'-',NOEpairsList.RemoteH(iNOEpairs,:)];
    NOEpeakList{count,4} = CS.CS(remoteH_idx);

    % pad residue numbers below 10 with leading zero to allow
    % sorting later on
    if centerResID < 10
        centerResIDstr = strcat('0',string(centerResID));
    else
        centerResIDstr = string(centerResID);
    end

    NOEpeakList{count,2} = strcat(centerRes,centerResIDstr,'-',centerHet);
    NOEpeakList{count,5} = CS.CS(centerHet_idx);
   
    NOEpeakList{count,3} = NOEpairsList.CenterH(iNOEpairs,:);
    NOEpeakList{count,6} = CS.CS(centerH_idx);
        
    % calculate NOE intensity based on proton-proton distance
    NOEpeakList{count,7} = NOEpairsList.NOEs(iNOEpairs,end);
    count = count + 1;
    
end

% trim empty cells
NOEpeakList = NOEpeakList(1:count-1,:);

% sort by second column, e.g. "V05-N"
[~,I] = sort(string(NOEpeakList(:,2)));
NOEpeakList = NOEpeakList(I,:);

% insert empty rows
lastRes = NOEpeakList{1,2};
emptyRow = cell(1,7);
NOEpeakList = [emptyRow;NOEpeakList];

nRes = size(unique(NOEpairsList.CenterResID),1);
for i = 2:size(NOEpeakList,1)+nRes-1
    currentRes = NOEpeakList{i,2};
    if ~strcmp(currentRes,lastRes)
        NOEpeakList = [NOEpeakList(1:i-1,:);emptyRow;NOEpeakList(i:end,:)];
        lastRes = currentRes;
    end
end

NOEpeakTable = table(NOEpeakList(:,1),...
                    NOEpeakList(:,2),...
                    NOEpeakList(:,3),...
                    NOEpeakList(:,4),...
                    NOEpeakList(:,5),...
                    NOEpeakList(:,6),...
                    NOEpeakList(:,7),...
                    'VariableNames',{'Assignment','Center Heteroatom','Center Proton','w1','w2','w3','Intensity'});

end

function dydt = odefun(t,y,R)
nPairs=size(R,1);
dydt = zeros(nPairs,1);
dydt(:) = -R(:,:)*y(:);
end

function d00 = dipolarC()
% calculate dipolar constant
gammaH = 2.6752e8; % per T per s
hbar = 1.055e-34; % J-s
mu0 = 1e-7; % T^2 m^3 per J

d00 = mu0^2 * gammaH^4 * hbar^2;
end