function [NOEpeakTable, R_all,NOE] = predictNOEmethyl_trj(trj,prmtop,CS,resTarget,proton,hetAtom,parms)
%NOEPEAKLIST produces predicts NOE peak list from an MD trajectory
%   Inputs
%           trj         MD trajectory coordinates, created using readmdcrd,
%                       mdtoolbox function
%
%           prmtop      topology file, created using readprmtop, mdtoolbox
%
%           CS          chemical shift struct, created using readShiftX
%
%           resTarget   sparse labeling residue(s), cell array
%
%           proton      atom name, defines center of sphere for NOE calcs
%                       e.g. 'H' for amide proton
%
%           parms       struct containing several parameters for the
%                       calculation
%
%               .radius     cutoff distance for NOE calculation
%                            e.g. 4, for atoms within 4 angstroms of center atom
%               
%               .tauC       rotational tumbling time, ns
%               
%               .tMax       maximum mixing time for NOE calculation, ms
%
%               .tStep      time step of MD trajectory frames, s
%
%               .Hfreq      Proton Larmor frequency, MHz
%
%               .ref_frame  reference frame for applying distance cutoff,
%               scalar double
%   Output
%           NOEpeakList     table containing all predicted NOE crosspeaks
%                           and diagonal peaks. Note peaks without a
%                           matching chemical shift are excluded. e.g. Arg
%                           HH
%
%           NOEpairsList    table containing all predicted NOE pairs
%                           for all requested protons and their distances.
%                           Unlike NOEpeakList no protons are ignored.  
%
%   Requirements
%           uses aminolookup function from bioinformatics toolbox

%% intialize noe prediction variables
wH = parms.Hfreq * 1e6 * 2*pi; % proton larmor frequency, rad per sec
% dd = 7.549e5; % dipolar constant
d00 = dipolarC();
maxFreq = 2*pi/(2*parms.tStep); 

tMix = linspace(0,parms.tMax)*1e-3; % seconds
tauC = parms.tauC*1e-9; % seconds

% internal correlation window size
nFrames = size(trj,1);
segSize = 3*tauC;
segFrames = floor(segSize/parms.tStep);
segHalf = floor(segFrames/2);
nSegs = floor(nFrames/segFrames);

if segFrames > nFrames
    warning('Trajectory is shorter than 3*tauC')
    segFrames = nFrames;
    segHalf = floor(segFrames/2);
    nSegs = 1;
end

% ensure at least 1000 points ultimately included in the spectral density
if segHalf < 1000
    segHalf = 1000;
    segFrames = segHalf * 2;
    nSegs = 1;
end

fprintf('Splitting trajectory into %d segments of %d frames...\n',nSegs,segFrames)

% indices to sample spectral density
freqStep = maxFreq/segHalf;
w2tp =ceil(2*wH/freqStep) + 1; 
wtp = ceil(wH/freqStep) + 1;

%% Figure out the spin systems

% find all hydrogen atoms, exlcuding those in water molecules
index_allH = find(selectname(prmtop.atom_name,'H*') & ~selectname(prmtop.residue_name,'WAT'));
allH = trj(parms.ref_frame,to3(index_allH));

% loop over residues
remoteH_id = [];
centerH_id = [];
invertedSpin = [];
for i = 1:numel(resTarget)
    % set atom names based on residue name
    if strcmp(resTarget{i},'VAL')
        nMethyl = 2;
        methylProtons = {{'HG11','HG12','HG13'},{'HG21','HG22','HG23'}};
        methylCarbons = {'C1','CG2'};
    elseif strcmp(resTarget{i},'LEU')
        nMethyl = 2;
        methylProtons = {{'HD11','HD12','HD13'},{'HD21','HD22','HD23'}};
        methylCarbons = {'CD1','CD2'};
    elseif strcmp(resTarget{i},'ILE')
        nMethyl = 2;
        methylProtons = {{'HG21','HG22','HG23'},{'HD11','HD12','HD13'}};
        methylCarbons = {'CG2','CD1'};
    elseif strcmp(resTarget{i},'MET')
        nMethyl = 1;
        methylProtons = {{'HE1','HE2','HE3'}};
        methylCarbons = {'CE'};
    elseif strcmp(resTarget{i},'ALA')
        nMethyl = 1;
        methylProtons = {{'HB1','HB2','HB3'}};
        methylCarbons = {'CB'};
    else
        error('selected residue does not contain a methyl group!')
    end
    
    % loop over methyls
    for j = 1:nMethyl
        % get coordinates for methyl protons
        currentProtons = methylProtons{j};
        index_center = find(selectname(prmtop.residue_name,resTarget{i}) & selectname(prmtop.atom_name,currentProtons{:}));
        
        % loop over methyl protons
        for k = 1:3:size(index_center,1)
            idx = k:k+2;
            centerAtoms = trj(parms.ref_frame,to3(index_center(idx)));
            avgMethyl = mean(reshape(centerAtoms,[3,3])');
            
            % find all other protons within cutoff distance
            [pair, dist] = searchrange_exhaustive(avgMethyl,allH,parms.radius);
            [pair, I] = sortrows(pair,2);
            dist = dist(I);
            
            % filter repeated atoms
            remote_tmp = prmtop.atom_id(index_allH(pair(:,1)));
            
            remoteH_id = [remoteH_id,{remote_tmp}];
            centerH_id = [centerH_id,{index_center(idx)}];            
        end
    end
end

%% Calculate Internal Correlation Functions
% this is a slow calculation, hours

% iterate over spin systems
nSystems = size(remoteH_id,2);
R_all = cell(1,nSystems);
totalPairs = 0;

NOE = cell(1,nSystems);
for i = 1:nSystems
    nPairs = size(remoteH_id{i},1);
    R = zeros(nPairs);
    d = zeros(nPairs);
    
    fprintf('Starting Residue %d, with %d spins\n',i,nPairs);
    tic
    % iterate over spin pairs
    for j = 1:nPairs
        for k = j+1:nPairs
            atom1_id = remoteH_id{i}(j);
            atom2_id = remoteH_id{i}(k);
            
            atom1 = trj(:,to3(atom1_id));
            atom2 = trj(:,to3(atom2_id));
            
            % skip pair if too far apart in first frame
            d(j,k) = norm(atom1(1,:) - atom2(1,:));
            if  d(j,k) > parms.radius
                continue
            end
            
            % start internal correlation function calculation
            % loop over subsegments
            c = zeros(segHalf,1); % correlation function for all segments            
            
            for segs = 1:nSegs
                currentFrames = (segs-1)*segFrames+1 : segs*segFrames;
                
                % interaction vector
                v = atom1(currentFrames,:) - atom2(currentFrames,:);

                c_seg = zeros(segHalf,1);
                for t = 1:segHalf
                    v_t = v(t,:) .* ones(segHalf,1);
                    r_t = norm(v_t(1,:));

                    tau = t + (1:segHalf);
                    v_tau = v(tau,:);
                    r_tau = vecnorm(v_tau,2,2);

                    cosTheta = dot(v_t,v_tau,2);
                    cosTheta = cosTheta / r_t;
                    cosTheta = cosTheta ./ r_tau;

                    p20 = (3*cosTheta.^2 - 1)/2;
                    p20 = p20 .* (r_t .* r_tau).^-3;

                    c_seg = c_seg + p20;
                end
                c = c + c_seg;
            end
            c = c/segHalf;
            c = c/nSegs;
            
            x = 1:segHalf;
            c = c' .* exp(-x*parms.tStep/tauC);
            J = dct(c,'Type',3);
            npts = length(J);
            J = J * pi / sqrt(2*npts) * npts / maxFreq;
            
            % now calculate relaxation rate matrix elements
            % equations from Cavanagh, pp377
            R(j,k) = (d00/10)*(-J(1)+6*J(w2tp)); % sigma
            R(k,j) = R(j,k);
            
            rho = (d00/10)*(J(1)+3*J(wtp)+6*J(w2tp));
            R(j,j) = R(j,j) + rho;
            R(k,k) = R(k,k) + rho;
        end
    end
    R_all{i} = R;
end
%% NOE calculation
% this is very fast, seconds
totalPairs = 1;

for i = 1:nSystems 
    nPairs = size(R_all{i},1);
    invertedSpin = zeros(3,1);
    for j = 1:3
        invertedSpin(j) = find(remoteH_id{i} == centerH_id{i}(j));
    end
    % define equilibrium state 
    Ieq = ones(nPairs,1);
    Iinit = Ieq;
    Iinit(invertedSpin) = -1;
    
    % Integrate system of differential equations
    y0 = (Iinit-Ieq)';
    [t,y] = ode45(@(t,y)odefun(t,y,R_all{i}),tMix,y0);
    NOE{i} = y';
    
    timeTake(i) = toc;
    totalPairs = totalPairs + nPairs;
end
fprintf('Finished in %.0f minutes %.0f seconds\n',sum(timeTake)/60,mod(sum(timeTake),60));

%% Match Predicted NOE crosspeaks with predicted chemical shifts and create peak list
NOEpeakList = cell(totalPairs,7);
NOEpeakList(:,7) = {0};
count = 1;

for i = 1:nSystems
    % rename atoms to match CS file
    RemoteRes = prmtop.residue_name(remoteH_id{i},:);
    RemoteH = prmtop.atom_name(remoteH_id{i},:);
    [RemoteRes,RemoteH] = messageMD(RemoteRes,RemoteH);
    
    remote_residue_id = prmtop.residue_id(remoteH_id{i});
    currentCenterAtom = remoteH_id{i}(remoteH_id{i} == centerH_id{i}(1));
    CenterH = prmtop.atom_name(currentCenterAtom,:);
    centerRes = aminolookup(strtrim(prmtop.residue_name(currentCenterAtom,:)));
    centerResID = prmtop.residue_id(currentCenterAtom);
    
    diagCount = 3;
    isDiag = false;
    for j = 1:size(RemoteH,1)
        res = aminolookup(strtrim(RemoteRes(j,:)));
        resNum = remote_residue_id(j);
        atom1 = strtrim(RemoteH(j,:));
        atom2 = strtrim(CenterH);
        
        if strcmp(atom1,atom2(1:end-1)) && ~isDiag
            isDiag = true;
            diagCount = 1;
        end
%         atom2 = atom2(1:3);
        
        remoteH_idx = selectname(CS.atomName,atom1) & CS.resID == resNum;
        centerH_idx = selectname(CS.atomName,atom2(1:end-1)) & CS.resID == centerResID;
        
        % determine appropriate heteronucleus, e.g. CG1 vs CG2
        for k = 1:size(methylProtons,2)
            for n = 1:size(methylProtons{k},2)
                if strcmp(atom2(1:3),methylProtons{k}{n}(1:3))
                    centerHet = hetAtom{k};
                end
            end
        end
        
        centerHet_idx = selectname(CS.atomName,centerHet) & CS.resID == centerResID;
        
        NOEpeakList{count,1} = [res,int2str(resNum),'-',RemoteH(j,:)];
        
        % check for missing shift prediction
        if sum(remoteH_idx) == 0
            NOEpeakList{count,4} = 999; % shift not found
        else
            NOEpeakList{count,4} = CS.CS(remoteH_idx);
        end
        
        % pad residue numbers below 10 with leading zero to allow
        % sorting later on
        if centerResID < 10
            centerResIDstr = strcat('00',string(centerResID));
        elseif centerResID < 100
            centerResIDstr = strcat('0',string(centerResID));
        else
            centerResIDstr = string(centerResID);
        end
        
        NOEpeakList{count,2} = strcat(centerRes,centerResIDstr,'-',centerHet);
        NOEpeakList{count,5} = CS.CS(centerHet_idx);
        
        NOEpeakList{count,3} = atom2;
        NOEpeakList{count,6} = CS.CS(centerH_idx);
        
        % calculate NOE intensity based on proton-proton distance
        intensity = NOE{i}(j,end);         
        
        if diagCount < 3
            NOEpeakList{count,7} = NOEpeakList{count,7} + intensity; 
            diagCount = diagCount + 1;
            continue
        else
            isDiag = false;
            NOEpeakList{count,7} = NOEpeakList{count,7} + intensity; 
        end
        
        count = count + 1;
        diagCount = 3;
    end
end
% trim empty cells
NOEpeakList = NOEpeakList(1:count-1,:);

% sort by second column, e.g. "V05-N"
[~,I] = sort(string(NOEpeakList(:,2)));
NOEpeakList = NOEpeakList(I,:);

% insert empty rows
lastRes = NOEpeakList{1,2};
emptyRow = cell(1,7);
NOEpeakList = [emptyRow;NOEpeakList];

% nRes = size(unique(CenterResID),1);
nRes = nSystems;
for i = 2:size(NOEpeakList,1)+nRes-1
    if i > size(NOEpeakList,1)
        break
    end
    currentRes = NOEpeakList{i,2};
    if ~strcmp(currentRes,lastRes)
        NOEpeakList = [NOEpeakList(1:i-1,:);emptyRow;NOEpeakList(i:end,:)];
        lastRes = currentRes;
    end
end

NOEpeakTable = table(NOEpeakList(:,1),...
                    NOEpeakList(:,2),...
                    NOEpeakList(:,3),...
                    NOEpeakList(:,4),...
                    NOEpeakList(:,5),...
                    NOEpeakList(:,6),...
                    NOEpeakList(:,7),...
                    'VariableNames',{'Assignment','Center Heteroatom','Center Proton','w1','w2','w3','Intensity'});

end

function dydt = odefun(t,y,R)
nPairs=size(R,1);
dydt = zeros(nPairs,1);
dydt(:) = -R(:,:)*y(:);
end

function d00 = dipolarC()
% calculate dipolar constant
gammaH = 2.6752e8; % per T per s
hbar = (6.626e-34)/(2*pi); % J-s
mu0 = 4*pi*1e-7; % T^2 m^3 per J

d00 = (mu0/(4*pi))^2 * gammaH^4 * hbar^2;% m^6 s^-2
d00 = d00 * 1e60; % convert m to Angstroms
end