function [preTable,r] = predictPRE(newPDB,resTarget,proton,I_cut,r0,Ln)
% predictPRE - single frame prediction of paramagnetic relaxation
% enhancement, part of the AssignSLPGUI package
%
% Inputs
%       newPDB
%       resTarget
%       proton
%       I_cut       intensity cutoff for observable PRE, 0.1 by default
%       r0          equivalent distance for I_cut
%       Ln          name of paramagnetic site, e.g. 'Dy3'
% Output
%       pre         predicted peak volume ratio, I_para / I_dia
%       r           H-Ln distances from PDB

% select atoms of interest
% resI = zeros(size(newPDB.resname,1),1);

para_id = selectname(newPDB.name,Ln); % find paramagnetic site
if size(para_id,1) == 0
    error('PredictNOE: Lanthanide not found!')
end

for i = 1:size(resTarget,2)
    if isempty(resTarget{i})
        continue
    end
    
    resI = selectname(newPDB.resname,resTarget{i});

    if resTarget{i} == "ILE" && proton{i} == "HG2"
        proton{i} = 'HG21';
    end
    
    proton_id = selectname(newPDB.name,proton{i}) & resI & (ismissing(cellstr(newPDB.altloc)) | selectname(newPDB.altloc,'A'));    
    
    r = sqrt(sum((newPDB.xyz(proton_id,:) - newPDB.xyz(para_id,:)).^2,2));
    
    % empirical prediction function proposed by JHP
    pre = exp(log(I_cut) .* (r0 ./ r).^6);
    pre(pre < 0.01) = 0;
    
    % make a pretty table
    resName = string(aminolookup(strtrim(newPDB.resname(proton_id,:)))');
    resNum = newPDB.resseq(proton_id);
    atomName = newPDB.name(proton_id,:);
    
    residue = strcat(resName,string(resNum),'-',atomName);
    vars = {'Residue','PRE Intensity ratio','Distance'};
    tmpTable = table(residue,pre,r,'VariableNames',vars);
    
    if i == 1
        preTable = tmpTable;
        sortMe = table(resName,resNum);
    else
        preTable = [preTable;tmpTable];
        sortMe = [sortMe;table(resName,resNum)];
    end
end

[~,I] = sortrows(sortMe);
preTable = preTable(I,:);
end

% created by RVW, 07/28/2021