function simNOESY = simulateNOESY(NOEsparky,cs_tol,I_tol)
%simulateNOESY  Creates simulated 3D-NOESY peak list by adding noise to the
%               chemical shift of NOE cross-peaks and to their intensity
%   Input:
%           NOEsparky   Predicted NOESY peak list, n x 7 cell array
%           cs_tol      Chemical Shift tolerance, magnitude of noise added
%                       to chemical shift of remote proton, scalar double
%           I_tol       Intensity tolerance, relative magnitude of noise added to
%                       intensity, e.g. +/- 10%, scalar double
%   Output:
%           simNOESY    Simulated NOESY peak list, n x 7 double
[rows1,cols1] = size(NOEsparky);

% remove empty rows
emptyCell = cellfun(@ismissing,NOEsparky,'UniformOutput',false);
emptyRows = cell2mat(emptyCell(:,4));
NOEsparky = NOEsparky(~emptyRows,:);
[rows,cols] = size(NOEsparky);

simNOESY = cell(rows,cols);

w1_col = 4; % chemical shift column to modify
w3_col = 6; % other shift for diagonal peak identification
int_col = 7; % intensity column

v = 1:cols;
v = v(v ~= w1_col & v~= int_col);
simNOESY(:,v) = NOEsparky(:,v);

% add noise
M = cell2mat(NOEsparky(:,[w1_col,int_col,w3_col]));
noise = [cs_tol, I_tol] .* randn(rows,2);

for i = 1:rows
    % check and skip diagonal peaks
    if M(i,1)==M(i,3)
        continue
    end
    
    M(i,1) = M(i,1) + noise(i,1);
    M(i,2) = M(i,2) + M(i,2) .* noise(i,2); % add error within +/- 10%
end

% output peak list
simNOESY(:,[w1_col, int_col]) = num2cell(M(:,1:2));

% put empty rows back
for i = 1:rows1
    if emptyRows(i)
        simNOESY = [simNOESY(1:i-1,:);cell(1,cols1);simNOESY(i:end,:)];
    end
end
end
