function simRDCs = simulateRDCs(RDCcoords,errorTol,SpinPair)
%simulateRDCs   Creates simulated RDCs within a specified error tolerance
%               based upon an arbitrary alignment tensor and supplied coordinates
%   Inputs:
%           RDCcoords   Coordinates from PDB or MD trajectory for spin
%                       pairs of interest, n x 8 cell array
%           errorTol    Error tolerance for RDC measurements, defines
%                       magnitude of noise, scalar double
%           SpinPair    Flag for spin pair type, 'NH' or 'Methyl'
%
%   Outputs:
%           simRDCs     Simulated RDCs, n x 3 cell array
%                       [peak number, RDC, ErrorTol]

nPeaks = size(RDCcoords,1);

% set appropriate Dmax and effective distance for spin pair
switch SpinPair
    case 'NH'
        Dmax = 24350; % Hz
        r_eff = 1.02; % idealized distance, Angstroms        
    case 'Methyl'
        Dmax = -2.675e8 *... % proton gyromagnetic ratio, rad / s / T        
                6.728e7 *... carbon-13 gyro        
                6.626e-34*...% planck, J sec
                1.2566e-6/... % u0, N A^-2
                (8*pi^3)*...
                1e30; % convert from m^3 to A^3, final units are Hz A^3
        Dmax = Dmax * (3*cosd(70.5)^2-1)/2; % additional scaling due to fast rotation and effective projection along C-C bond axis
        r_eff = 1.1;
end

atom_xyz = cell2mat(RDCcoords(:,2:7));

% make bond vectors
bond_xyz = atom_xyz(:,1:3) - atom_xyz(:,4:end);

% build A matrix (equations from REDCAT paper) containing angular terms
A = [bond_xyz(:,2).^2 - bond_xyz(:,1).^2, ... % y2 - x2
     bond_xyz(:,3).^2 - bond_xyz(:,1).^2, ... % z2 - x2
     2*bond_xyz(:,1).*bond_xyz(:,2),... % 2xy
     2*bond_xyz(:,1).*bond_xyz(:,3),... % 2xz
     2*bond_xyz(:,2).*bond_xyz(:,3)]; % 2yz

% scale by distance and Dmax
A = Dmax .* r_eff.^-5 .* A; % scale A
 
% define arbitrary alignment tensor, assume coordinates are already in
% principal coordinate system
Szz = 4e-4;
Syy = -Szz / 2; % axial symmetry
tensor = [Syy, Szz, 0, 0, 0]; % arbitrary tensor, assume coords are in principal axis system

D_calc = A * tensor'; % calculate RDCs
D_calc = D_calc + errorTol .* randn(nPeaks,1);

simRDCs = num2cell( [[1:nPeaks]', D_calc, errorTol*ones(nPeaks,1)]);

end

