%% sparky2strips
% created by RVW - 9/20/2019
% convert Sparky format peaklist into a matrix of NOE vectors

% Inputs
%     SparkyCell - cell array containing Sparky format peak list for 3D
%       NOESY-HSQC experiment (t1,t2,t3) is (1H,15N,1H)
%           {Assignment - w1} {Assignment - w2} {Assignment - w3} {w1 (ppm)} {w2 (ppm)} {w3
%           ppm)}
%   
%     gaussWidth - scalar defining the width of the each NOE cross-peak (in
%          ppm)
    
% Output
%     NOEstrips - num array containing NOE vectors in columns

function [NOEstrips,peaks] = sparky2strips(SparkyCell,gaussWidth)

[rows,~] = size(SparkyCell);
intCol = 7; % column with Intensity information
noeCol = 4; % column with NOESY peak ppm, w1 1H
diagTol = 0.01; % should be pretty small.

% initialize some dummy variables
count = 1; % count of the vector number
pCount = 1; % count of peaks in current vector

% define chemical shift range
npts = 512;
x = linspace(-1,12,npts);

%% break peak list into 3D cell array
% each plane (dim1, dim2)is a peak list for one NOE strip
for i = 1:rows    
    if i == 1 % initialize first vector
        % create list of peaks for the current vector
        peaks(pCount,:,count) = SparkyCell(i,:);
        
    elseif ~ismissing(SparkyCell{i,5})
        % append peak to vector
        pCount = pCount + 1;
        peaks(pCount,:,count) = SparkyCell(i,:);
        
    else
        % Reset variables for a new vector
        count = count + 1; % increment vector counter
        pCount = 1;
        if ~ismissing(SparkyCell{i,5})
            for j = 1:size(SparkyCell,2)
                peaks{pCount,j,count} = SparkyCell{i,j};
            end
        end
    end   
end

%% create NOE strips
M = zeros(npts,count);
idx_missing = zeros(count,1);
for i = 1:count
    currentPeakList = peaks(:,:,i);
    % remove empty rows
    idx_empty = zeros(size(currentPeakList,1),1);
    for j = 1:size(currentPeakList,1)
        if isempty(currentPeakList{j,7})
            idx_empty(j) = 1;
        end
    end
    
    currentPeakList(logical(idx_empty),:) = [];
    
    if currentPeakList{1,end} == 999
        % missing NOE strip
        idx_missing(i) = true;
        continue
    end
    
    
    % find and remove diagonal peak
    [~,maxI] = max(cell2mat(currentPeakList(:,7))); % most intense peak probs diagonal
    if abs(currentPeakList{maxI,4} - currentPeakList{maxI,6}) < diagTol % verify actually diagonal
       currentPeakList(maxI,:) = cell(1,7); % replace row with empty cell
    end

    % convert vector peak list into gaussians
    npeaks = size(currentPeakList,1);
    vector = zeros(npts,1);


    for j = 1:npeaks
        if isempty(currentPeakList{j,noeCol})            
            if npeaks == 1
                % only observed diagonal peak, no informative peaks
                idx_missing(i)=true;
                break
            end
            continue
        else
            gaussian = double(currentPeakList{j,intCol})*exp(-4*log(2).*(x - double(currentPeakList{j,noeCol})).^2/(gaussWidth^2));
            vector = vector + gaussian';
        end
    end

    % add NOEvector to output matrix
    M(:,i) = vector;
end
%% Add Pseudopeaks

% Pseudopeak intensity is the average of the largest peak in each vector
height = mean(max(M));
[~,cols] = size(M);

% create pseudopeak vector
pseudopeak = height*exp(-4*log(2).*(x - x(end)).^2/(gaussWidth^2));

% expand to a matrix of same dimensions as NOE vector matrix
pseudopeakM = pseudopeak' * ones(1,cols);

% combine matrices
NOEstrips = M + pseudopeakM;

%% Add missing vectors as cols of 999
NOEstrips(:,logical(idx_missing)) = 999*ones(npts,1)*ones(1,sum(idx_missing));
end
